
#include "includes.h"

/*
********************************************************************************************************************
*												Pragma Section
*
* Configuration bits under MPLAB v7.60 subject to changes on every little change on project settings.
* So #pragma config statments used for configuration bits instead of using Configuration bits tab in MPLAB
* Refer to pdf document "mcc18\doc\MPLAB-C18-Addendum.pdf" under mcc18 installation directory for config
* variables available
********************************************************************************************************************
*/
#pragma config PLLDIV = 5			// 20Mhz crystal input, the PLL divider has to be 5 for a fixed 4MHz input to PLL
#pragma config FOSC   = HSPLL_HS	// HS oscillator, PLL sourced
#pragma config CPUDIV = OSC1_PLL2	// Since it is now PLL sourced, the CPU clock = 96MHz PLL/2 => 48MHz
									// From specification the max. clock is around 20MHz for Vdd=3.3V
									// This scenario is like a bit over-clocking the microcontroller!
#pragma config PWRT   = ON			// PowerUp timer ON
#pragma config WDT    = OFF			// watchdog timer off
#pragma config LVP    = OFF			// low level voltage progamming OFF
#pragma config BOR    = OFF			// brown out detect off
#pragma config DEBUG  = ON			// Debug ON
#pragma config PBADEN = OFF 		// PORTB<4:0> as digital IO on reset
#pragma config STVREN = ON  		// Stack overflow reset ON
#pragma config VREGEN = OFF			// Disable internal 3.3V regulator!

///////////////////////////////// FONTS ///////////////////////////////////////
extern FONT_FLASH Font25;

rom char stringTemp[]  = "Temperature:";
rom char stringRH[]    = "Relative Humidity:";

typedef union
{
	unsigned int i;
	float f;
} value;



/*
*********************************************************************************************************
*                         				CALCULATE RH AND TEMPERATURE
*
* Description :	This function returns temperature and relative humidity in floating point values
* Arguments   : float '*p_humidity'		pointer to humidity
*				float '* p_temperature' pointer to temperature
*				
* Returns     : none
* Notes		  : humidity in 12 bit raw data and temperature in 14 bit raw data passed to this
*				function, where at the same ram location, floating point values of temperature and
*				relative humidity value will be returned. 
*				Reference: an Keil C example for 8051 from Sensirion at www.sensirion.com
*********************************************************************************************************
*/	

void calc_sht1x(float *p_humidity ,float *p_temperature)
// calculates temperature [C] and humidity [%RH] 
// input :  humi [Ticks] (12 bit) 
//          temp [Ticks] (14 bit)
// output:  humi [%RH]
//          temp [deg Cel]
{ float rh=*p_humidity;             				// rh:      Humidity [Ticks] 12 Bit 
  float t=*p_temperature;           				// t:       Temperature [Ticks] 14 Bit
  float rh_lin;                     				// rh_lin:  Humidity linear
  float rh_true;                    				// rh_true: Temperature compensated humidity
  float t_C;                        				// t_C   :  Temperature [C]

  t_C=t*0.01 - 40;                  				//calc. temperature from ticks to [deg Cel]	
  rh_lin=0.0405*rh - 0.0000028*rh*rh - 4.0;     	//calc. humidity from ticks to [%RH]
  rh_true=(t_C-25)*(0.01+0.00008*rh)+rh_lin;   		//calc. temperature compensated humidity [%RH]
  if(rh_true>100)rh_true=100;       				//cut if the value is outside of
  if(rh_true<0.1)rh_true=0.1;       				//the physical possible range

  *p_temperature=t_C;               				//return temperature [deg Cel]
  *p_humidity=rh_true;              				//return humidity[%RH]
}

/*
*********************************************************************************************************
*                        CONVERT A FLOATING POINT NUMBER TO STRING WITH 1 DECIMAL PLACE
*
* Description :	This function converts a floating point to a null terminated string 
*				with 1 decimal place.
*			
*				Examples:
*					
*				float f = 9.567;
*				ftoa(&s[0], f);		//s[]={'9','.','5', 0}
*				float f = -0.189;
*				ftoa(&s[0], f);  	//s[]={'-', '0', '.', '1', 0}
* Arguments   : 'unsigned char* buf' 	is the pointer to the string holding the conversion result
*				'float f'				is the input floating point
* Returns     : Returns the string with unsigned char* buf pointing to.
* Notes		  : This routine modified from itoa10() in ..\sample\misc folder of ht-picc
*				If more decimal places required, modify the last section of this code
*				Range of f in (-3,276.7, 3,276.7)
*				This function does print result like 0.0.
*********************************************************************************************************
*/
void ftoa(unsigned char *buf, float f) {
	unsigned int rem;
	unsigned char *s,length=0;
	int i;

	i = (int)((float)f*10);

	s = buf;
	if (i == 0){ 		//print 0.0 with null termination here
		*s++ = '0';
		*s++ = '.';
		*s++ = '0'; 
		*s=0; 			//null terminate the string
	} else {	
		if (i < 0) {
			*buf++ = '-';
			s = buf;
			i = -i;
		}
		//while-loop to "decode" the long integer to ASCII by append '0', string in reverse manner
		//If it is an integer of 124 -> string = {'4', '2', '1'}
		while (i) {
			++length;
			rem = i % 10;
			*s++ = rem + '0';
			i /= 10;
		}
		//reverse the string in this for-loop, string became {'1', '2', '4'} after this for-loop
		for(rem=0; ((unsigned char)rem)<length/2; rem++) {
			*(buf+length) = *(buf+((unsigned char)rem));
			*(buf+((unsigned char)rem)) = *(buf+(length-((unsigned char)rem)-1));
			*(buf+(length-((unsigned char)rem)-1)) = *(buf+length);
		}

		/* Take care of the special case of 0.x if length ==1*/	
		if(length==1) {
			*(buf+2) = *buf;
			*buf = '0';
			*(buf+1) = '.';
			*(s+2)=0; 			//null terminate
		} else {
			*(buf+length) = *(buf+length-1);
			*(buf+length-1)='.';
			*(s+1)=0; 			//null terminate
		}
	}
}

/*
********************************************************************************************************************
*													MAIN
********************************************************************************************************************
*/
void main(void)
{
// Sample program that shows how to use SHT10 functions and got the data displayed on 2.4" TFT color LCD
// 1. Initialize the graphics interface by InitGraph().
// 2. Set font to Font25
// 3. Connection reset (sensor init)
// 4. Measure humidity [ticks](12 bit) and temperature [ticks](14 bit)
// 5. Calculate humidity [%RH] and temperature [C]
// 6. Convert temperature in floating point to null terminated string for printing
// 7. Clear the background for digits
// 8. Print the digits of temperature
// 9. Repeat for RH value
// 10. Delay for 1s and loop all over

	value humi_val, temp_val;
	unsigned char error, checksum;
	unsigned char s[20];
	unsigned char i;
	
	ADCON1 = 0x0F;					//All digital operation	for RA2 & RA0 pins being digital for SHT10 sensor

	InitGraph();					//initialize the Graphics library
	SetFont(&Font25);	
	SetColor(BLACK);
	ClearDevice();
	SetColor(BRIGHTGREEN);
	
	for(i=0;i<20;i++) s[i] = 0; i = 0;
	while(stringTemp[i]) {s[i] = stringTemp[i]; i++;};
	OutTextXY(10,160,&s[0]);	

	for(i=0;i<20;i++) s[i] = 0; i = 0;
	while(stringRH[i]) {s[i] = stringRH[i]; i++;};	
	OutTextXY(10,280,&s[0]);	

	s_softreset();					//reset the SHT10 sensor
	DelayMs(20);

	for(;;){
	error=0;
    error+=s_measure((unsigned char*)&humi_val.i,&checksum,HUMI);  	//measure humidity
    error+=s_measure((unsigned char*)&temp_val.i,&checksum,TEMP);  	//measure temperature
    if(error!=0) s_connectionreset();								//in case of an error: connection reset               
    else
	{
      	humi_val.f=(float)humi_val.i;								//converts integer to float
      	temp_val.f=(float)temp_val.i;								//converts integer to float
      	calc_sht1x(&humi_val.f,&temp_val.f);						//calculate humidity, temperature
		SetColor(DARKGRAY);
		FillBevel(178,160,220,185,1);								//redraw bevel definition for Temperature reading
		FillBevel(178,280,220,305,1);								//redraw bevel definition for RH reading
		//print temperature string
		ftoa(s, temp_val.f);										//convert floating point to string in 1 dec place
		SetColor(BRIGHTGREEN);
        OutTextXY( 180,160,&s[0]);								
		//print RH string
		ftoa(s, humi_val.f);										//convert floating point to string in 1 dec place
		OutTextXY( 180,280,&s[0]);						
    }
      //----------wait approx. 1s to avoid heating up SHTxx--------------    
	 DelayMs(1000);       
	}
}

